let Generator = require('aws-generator'),
    AWS = Generator.AWS,
    uuid = Generator.NodeUUID;
function S3WrapperError(error) {
    this.error = error;
    this._id = uuid;
    this._at = new Date();
}

function S3Wrapper(s3, params, type) {
    let _this = this;
    if (s3 && !s3.Bucket && s3.config){
        this.s3 = s3
        this.params = params;
        this.type = params;
    } else if (s3 && !params && s3.Bucket && s3.Key){
        this.s3 = new AWS.S3({apiVersion: "2006-03-01", region: process.env.region})
        this.params = s3;
    } else {
    }
}

S3Wrapper.prototype.get = function (callback) {
    if (callback) {
        if (!this.params || !this.params.Bucket || !this.params.Key) return callback(new S3WrapperError("Invalid Parameters"));
        this.s3.getObject(this.params, async function (err, got) {
            if (err) return callback(new S3WrapperError(err));
            try {
                let bod = got.Body.toString(), JSONResult;
                if (!bod) return callback(Promise.reject(new S3WrapperError('ERROR: Failed to Parse Data')));
                JSONResult = await JSON.parse(bod);
                return callback(null, JSONResult);
            } catch(e) {
                return callback(Promise.reject(new S3WrapperError(e)));
            }
        })
    } else {
        let it = this;
        return new Promise(function (result) {
            if (!it.params || !it.params.Bucket || !it.params.Key) return result(Promise.reject(new S3WrapperError("Invalid Parameters")));
            it.s3.getObject(it.params, async function (err, got) {
                if (err) return result(Promise.reject(new S3WrapperError(err)));
                try {
                    let bod = got.Body.toString(), JSONResult;
                    if (!bod) return result(Promise.reject(new S3WrapperError('ERROR: Failed to Parse Data')));
                    JSONResult = await JSON.parse(bod);
                    return result(Promise.resolve(JSONResult));
                } catch(e) {
                    return result(Promise.reject(Promise.reject(new S3WrapperError(e))));
                }
            })
        })
    }
}

module.exports = S3Wrapper;